@@ -1,5 +1,5 @@
 //    XSDDiagram - A XML Schema Definition file viewer
-//    Copyright (C) 2006  Regis COSNIER
+//    Copyright (C) 2006-2011  Regis COSNIER
 //
 //    This program is free software; you can redistribute it and/or modify
 //    it under the terms of the GNU General Public License as published by
@@ -15,14 +15,10 @@
 //    along with this program; if not, write to the Free Software
 //    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-using System;
-using System.Collections.Generic;
-using System.Text;
 
 namespace XMLSchema
 {
-    using System.Xml.Serialization;
-	using System.ComponentModel;
+    using System.ComponentModel;
 
 	[TypeConverter(typeof(ExpandableObjectConverter))]
 	partial class annotation
